"""
Top-level package for InfraHouse Toolkit.

:Author: Oleksandr Kuzminskyi
:Version: 2.9.4
:Email: aleks@infrahouse.com
"""

__author__ = """Oleksandr Kuzminskyi"""
__email__ = "aleks@infrahouse.com"
__version__ = "2.9.4"

from logging import getLogger

DEFAULT_OPEN_ENCODING = "utf8"
DEFAULT_ENCODING = DEFAULT_OPEN_ENCODING
LOG = getLogger(__name__)
